@@ -1,6 +1,5 @@
 #' Estimates an unbiased VAR(1) using stochastic approximation (Bauer, Rudebusch and Wu, 2012)
 #'
-#'
 #'@param ModelType A character vector indicating the model type to be estimated.
 #'@param BRWinputs A list containing the necessary inputs for the BRW model estimation:
 #'\enumerate{
@@ -21,8 +20,6 @@
 #'@param ev_restr Numeric. Restriction on the largest eigenvalue under the P-measure. Default is 1.
 #'@param nargout Integer. Number of elements in the output list. Default is 4.
 #'
-#'
-#'
 #'@examples
 #'\donttest{
 #' data(CM_Factors)
@@ -36,11 +33,10 @@
 #' ModelType <- "JPS original"
 #' FactorLabels <- NULL
 #'
-#'
 #' BRWpara <- Bias_Correc_VAR(ModelType, BRWinputs, Factors, N, Economies, FactorLabels)
 #'}
 #'
-#'@return Bias-corrected VAR paramaters based on the framework of Bauer, Rudebusch and Wu (2012). The list contains:
+#'@return Bias-corrected VAR parameters based on the framework of Bauer, Rudebusch and Wu (2012). The list contains:
 #'\enumerate{
 #'\item \code{Phi_tilde}: estimated coefficient matrix (F x F);
 #'\item \code{mu_tilde}: estimated intercept (F x 1);
@@ -50,15 +46,12 @@
 #'      reported if nargout is 5.
 #'}
 #'
-#'
 #'@references
 #' Bauer, Rudebusch and, Wu (2012). "Correcting Estimation Bias in Dynamic Term Structure Models" \cr
 #' This function is based on the \code{est_unb_var} Matlab function available at Cynthia Wu's website
 #' (https://sites.google.com/view/jingcynthiawu/).
 #'@export
 
-
-
 Bias_Correc_VAR  <- function(ModelType, BRWinputs, RiskFactors, N, Economies, FactorLabels, GVARinputs = NULL,
                              JLLinputs = NULL, ev_restr = 1, nargout= 4){
 
@@ -80,14 +73,14 @@ Bias_Correc_VAR  <- function(ModelType, BRWinputs, RiskFactors, N, Economies, Fa
 
   # 2) initialization for SA algorithm
   theta <- matrix(0, nrow = k^2, ncol=N_burn+N_iter)
-  theta_hat <- Phi_hat[1:(ncol(Phi_hat)*nrow(Phi_hat))]
+  theta_hat <- as.vector(Phi_hat)
   theta[ ,1] <- theta_hat # starting value
 
   # 3) SA algorithm
   for (j in 1:(N_burn+N_iter-1)){
     Phi_new <- m_var(theta[ ,j], B, RiskFactors, N, GVARinputs, JLLinputs, FactorLabels, Economies,
                      ModelType, flag_mean)$Phi_new
-    theta_new <- c(Phi_new)
+    theta_new <- as.vector(Phi_new)
     d <- theta_hat - theta_new
     theta[ ,j+1] <- theta[ ,j] + gamma*d
 
@@ -96,7 +89,7 @@ Bias_Correc_VAR  <- function(ModelType, BRWinputs, RiskFactors, N, Economies, Fa
       theta_tilde <- rowMeans(theta[ ,(N_burn+1):j])
       Phi_tilde <- matrix(theta_tilde,k,k)
       cat(paste('  *** Iteration:', j-N_burn, '. Largest absolute eigenvalue: ',
-                round(max(abs(eigen(Phi_tilde)$value)), 6),"\n"))
+                round(max(abs(eigen(Phi_tilde)$values)), 6),"\n"))
     }
   }
 
@@ -120,18 +113,15 @@ Bias_Correc_VAR  <- function(ModelType, BRWinputs, RiskFactors, N, Economies, Fa
   mu_tilde <- (diag(k) - Phi_tilde)%*% t(t(colMeans(RiskFactors)))
 
   # 8) residuals and their variance-covariance matrix
-  if(checkSigma){
-  V_tilde <- Get_V_tilde_BC(Phi_tilde, N, RiskFactors, GVARinputs, JLLinputs, FactorLabels, ModelType)
-  }else{ V_tilde <- NaN}
+  V_tilde <- if (checkSigma) Get_V_tilde_BC(Phi_tilde, N, RiskFactors, GVARinputs, JLLinputs, FactorLabels, ModelType) else NaN
 
   # 9) Compilled desired outputs
-  if (nargout==5){
-    output <- list( Phi_tilde = Phi_tilde, mu_tilde = mu_tilde, V_tilde = V_tilde, dist = dist, Phi_sample= Phi_sample)
-    }else{output <- list(Phi_tilde = Phi_tilde, mu_tilde = mu_tilde, V_tilde = V_tilde, dist = dist) }
+  output <- list(Phi_tilde = Phi_tilde, mu_tilde = mu_tilde, V_tilde = V_tilde, dist = dist)
+  if (nargout == 5) output$Phi_sample <- Phi_sample
 
   return(output)
 }
-###############################################################################################################
+
 ###############################################################################################################
 #' Estimate a VAR(1) - suited to Bauer, Rudebusch and Wu (2012) methodology
 #'
@@ -158,71 +148,44 @@ Bias_Correc_VAR  <- function(ModelType, BRWinputs, RiskFactors, N, Economies, Fa
 #' This function is similar to the "estVAR" Matlab function available at Cynthia Wu's website
 #' (https://sites.google.com/view/jingcynthiawu/).
 
-
-
 estVARbrw <- function(RiskFactors, ModelType, N, GVARinputs, JLLinputs, FactorLabels, Economies,
                       demean = FALSE, intercept = TRUE){
 
-
   T <- dim(RiskFactors)[1]; k <- dim(RiskFactors)[2]
 
   # 1) Data prep
-  if (demean == TRUE){  RiskFactors <- RiskFactors - matrix(1, T, 1)%*%colMeans(RiskFactors) }
+  if (demean) RiskFactors <- scale(RiskFactors, scale = FALSE)
 
   Yt <- RiskFactors[1:(T-1), ]  # (T-1)*k
   Ytp1 <- RiskFactors[2:T, ]  # (T-1)*k
   Y <- t(Ytp1)  # k*(T-1)
 
-    if (intercept== TRUE){Z <- t(cbind(matrix(1, T-1, 1), Yt)) # (k+1)*(T-1)
-    }else{  Z <- t(Yt)} # k*(T-1)
+  Z <- if (intercept) t(cbind(1, Yt)) else t(Yt) # (k+1)*(T-1) or k*(T-1)
 
   # 2) Generate feedback matrix of a VAR(1) with demeaned data
-    # a) JPS-related models
-    if(any(ModelType == c("JPS original", "JPS global", "JPS multi"))){
-    A <- Y%*%t(Z)%*%solve(Z%*%t(Z), tol = 1e-50) # k*(k+1) / k*k
-  if (intercept== TRUE){
-    alpha_hat = A[ ,1]
-    Gamma_hat = A[,2:ncol(A)]
-  }else{
-    alpha_hat <- NaN
-    Gamma_hat <- A
-  }
+  if (any(ModelType == c("JPS original", "JPS global", "JPS multi"))){
+    A <- Y %*% t(Z) %*% solve(Z %*% t(Z), tol = 1e-50) # k*(k+1) / k*k
+    alpha_hat <- if (intercept) A[, 1] else NaN
+    Gamma_hat <- if (intercept) A[, -1] else A
 
-
-  # b) GVAR-related models
-  }else if(any(ModelType == c("GVAR single", "GVAR multi")))
-  {
-  Economies <- GVARinputs$Economies # necessary to ensure that the code works for the "GVAR sepQ" model
-  GVARinputs$GVARFactors <- DataSet_BS(ModelType, t(RiskFactors), GVARinputs$Wgvar, Economies, FactorLabels)
-  GVARpara <- GVAR(GVARinputs, N)
-  if (intercept== TRUE){
-    alpha_hat <- GVARpara$F0
-    Gamma_hat <-GVARpara$F1
-  }else{
-    alpha_hat <- NaN
+  } else if (any(ModelType == c("GVAR single", "GVAR multi"))){
+    Economies <- GVARinputs$Economies # necessary to ensure that the code works for the "GVAR sepQ" model
+    GVARinputs$GVARFactors <- DataSet_BS(ModelType, t(RiskFactors), GVARinputs$Wgvar, Economies, FactorLabels)
+    GVARpara <- GVAR(GVARinputs, N)
+    alpha_hat <- if (intercept) GVARpara$F0 else NaN
     Gamma_hat <- GVARpara$F1
-  }
-  # c) JLL-related models
-  } else if(any(ModelType == c("JLL original", "JLL No DomUnit", "JLL joint Sigma")))
-  {JLLinputs$WishSigmas <- 0
-  JLLPara <- JLL(t(RiskFactors), N, JLLinputs)
-    if (intercept== TRUE){
-    alpha_hat <- JLLPara$k0
-    Gamma_hat <-JLLPara$k1
-  }else{
-    alpha_hat <- NaN
+
+  } else if (any(ModelType == c("JLL original", "JLL No DomUnit", "JLL joint Sigma"))){
+    JLLinputs$WishSigmas <- 0
+    JLLPara <- JLL(t(RiskFactors), N, JLLinputs)
+    alpha_hat <- if (intercept) JLLPara$k0 else NaN
     Gamma_hat <- JLLPara$k1
   }
-  }
 
-
-  Output <- list(Gamma_hat = Gamma_hat, alpha_hat = alpha_hat)
-
-  return(Output)
+  return(list(Gamma_hat = Gamma_hat, alpha_hat = alpha_hat))
 }
 
-###################################################################################################################
-###################################################################################################################
+###############################################################################################################
 #' Generate M data sets from VAR(1) model
 #'
 #'@param Phi feedback matrix (F x F)
@@ -236,39 +199,33 @@ estVARbrw <- function(RiskFactors, ModelType, N, GVARinputs, JLLinputs, FactorLa
 #' This function is based on to the"genVAR" Matlab function available at Cynthia Wu's website
 #' (https://sites.google.com/view/jingcynthiawu/).
 
-
 genVARbrw <- function(Phi, M, RiskFactors){
 
-
   T <- dim(RiskFactors)[1];  k <- dim(RiskFactors)[2]
 
   Y_mean <- colMeans(RiskFactors)
   Y_mean_rep <- t(t(Y_mean))%*%matrix(1,1,M)
   X_sim <- array(0, c(k, T, M))
 
-    # VAR(1)
-    # 1. obtain residuals
-    # use bootstrapped residuals
-    resid <- matrix(0,k, T-1)
-    for (t in 2:T){ resid[,(t-1)] <- t(t(RiskFactors[t, ])) -t(t(Y_mean)) - (Phi %*% t(t(RiskFactors[t-1, ] - Y_mean)))}
+  # VAR(1)
+  # 1. obtain residuals
+  resid <- apply(RiskFactors, 1, function(row) row - Y_mean - Phi %*% (row - Y_mean))
 
-    # 2. generate series
-    # randomly select initial values from the data Y
-    ind_start <- sample(T, M, replace = TRUE)
-    X_sim[ ,1, ] <- t(RiskFactors[ind_start, ])
-    for (t in 2:T){
-      u_sim <- resid[ , sample(T-1, M, replace = TRUE)]
-      X_sim[ ,t, ] <- Y_mean_rep + Phi%*% (drop(X_sim[ ,t-1,]) - Y_mean_rep) + u_sim
-    }
+  # 2. generate series
+  # randomly select initial values from the data Y
+  ind_start <- sample(T, M, replace = TRUE)
+  X_sim[, 1, ] <- t(RiskFactors[ind_start, ])
+  for (t in 2:T){
+    u_sim <- resid[ , sample(T-1, M, replace = TRUE)]
+    X_sim[ ,t, ] <- Y_mean_rep + Phi%*% (drop(X_sim[ ,t-1,]) - Y_mean_rep) + u_sim
+  }
 
-
-    rownames(X_sim) <- colnames(RiskFactors)
+  rownames(X_sim) <- colnames(RiskFactors)
 
   return(X_sim)
 }
 
-###################################################################################################################
-###################################################################################################################
+###############################################################################################################
 #' Find mean or median of OLS when DGP is VAR(1)
 #'
 #'@param theta parameters from the feedback matrix in vector form
@@ -288,47 +245,36 @@ genVARbrw <- function(Phi, M, RiskFactors){
 #' Bauer, Rudebusch and, Wu (2012). "Correcting Estimation Bias in Dynamic Term Structure Models". \cr
 #' This function is similar to the "m_var" Matlab function available at Cynthia Wu's website
 #' (https://sites.google.com/view/jingcynthiawu/).
-#'
 
 m_var <- function(theta, M, RiskFactors, N, GVARinputs, JLLinputs, FactorLabels, Economies, ModelType, flag_mean = TRUE){
 
   T <- dim(RiskFactors)[1]; k <- dim(RiskFactors)[2]
 
-    if (flag_mean== TRUE){  Phi_sample <- array(0,c(k,k,M))}else{ Phi_sample <- NaN }
+  Phi_sample <- if (flag_mean) array(0, c(k, k, M)) else NaN
 
-    # 1) get coefficient matrix
-    Phi_tilde <-  matrix(theta, k, k)
+  # 1) get coefficient matrix
+  Phi_tilde <- matrix(theta, k, k)
 
-    # 2) simulate M datasets
-    X_sim <- genVARbrw(Phi_tilde, M, RiskFactors)
+  # 2) simulate M datasets
+  X_sim <- genVARbrw(Phi_tilde, M, RiskFactors)
 
-    # 3) estimate VAR(1) on each series
-    theta_new_i <- matrix(0, M, k^2)
-    for (m in 1:M){
-      Phi_new <- estVARbrw(t(X_sim[ , ,m]), ModelType, N, GVARinputs, JLLinputs, FactorLabels, Economies,
-                           demean = TRUE, intercept = FALSE)$Gamma_hat
+  # 3) estimate VAR(1) on each series
+  theta_new_i <- matrix(0, M, k^2)
+  for (m in 1:M){
+    Phi_new <- estVARbrw(t(X_sim[ , ,m]), ModelType, N, GVARinputs, JLLinputs, FactorLabels, Economies,
+                         demean = TRUE, intercept = FALSE)$Gamma_hat
 
-      if (flag_mean == TRUE){ Phi_sample[ , , m] <- Phi_new }
-      theta_new_i[m, ] <- c(Phi_new) # All elements as a vector
-      }
+    if (flag_mean) Phi_sample[ , , m] <- Phi_new
+    theta_new_i[m, ] <- as.vector(Phi_new) # All elements as a vector
+  }
 
-    # 4) Find mean/median of OLS estimates
-    if (flag_mean== TRUE){
-      # Mean
-      Phi_new <- matrix(colMeans(theta_new_i), k, k )
-    } else{
-      # Median
-      Phi_new <- matrix(apply(theta_new_i, 2 , stats::median), k, k )
-    }
+  # 4) Find mean/median of OLS estimates
+  Phi_new <- if (flag_mean) matrix(colMeans(theta_new_i), k, k) else matrix(apply(theta_new_i, 2, stats::median), k, k)
 
-
-  Outputs <- list(Phi_new = Phi_new, Phi_sample = Phi_sample)
-
-  return(Outputs)
+  return(list(Phi_new = Phi_new, Phi_sample = Phi_sample))
 }
 
-###################################################################################################################
-###################################################################################################################
+###############################################################################################################
 #' Killan's VAR stationarity adjustment
 #'
 #'@param Phi_tilde VAR (1) bias-corrected feedback matrix from Bauer, Rudebusch and, Wu (2012)
@@ -344,35 +290,32 @@ m_var <- function(theta, M, RiskFactors, N, GVARinputs, JLLinputs, FactorLabels,
 #' Bauer, Rudebusch and, Wu (2012). "Correcting Estimation Bias in Dynamic Term Structure Models". \cr
 #' This function is an adapted version of the"shrink_Phi" Matlab function available at Cynthia Wu's website
 #' (https://sites.google.com/view/jingcynthiawu/).
-#'
 
 shrink_Phi <- function(Phi_tilde, Phi_hat, ev_restr){
 
+  ev_bc <- max(abs(eigen(Phi_tilde)$values))   # get scalar largest absolute eigenvalues
+  ev_ols <- max(abs(eigen(Phi_hat)$values))
 
-  ev_bc <- max(abs(eigen(Phi_tilde)$value))   # get scalar largest absolute eigenvalues
-  ev_ols <- max(abs(eigen(Phi_hat)$value))
-
-  if (ev_bc>ev_restr){
-    if (ev_ols<ev_restr){ # if Phi_hat actually satisfies eigenvalue restriction, then then shrink Phi_tilde to Phi_hat
-  Phi_diff0 <- Phi_hat - Phi_tilde
-  delta <- 1
-  while ((ev_bc > ev_restr) & delta > 0){ # delta > 0 is necessary to ensure that Phi_tilde won't get explosive
-    delta <- delta - 0.01
-    Phi_diff <- delta*Phi_diff0
-    Phi_tilde <- Phi_hat - Phi_diff
-    ev_bc <- max(abs(eigen(Phi_tilde)$value))
-    }
+  if (ev_bc > ev_restr){
+    if (ev_ols < ev_restr){ # if Phi_hat actually satisfies eigenvalue restriction, then shrink Phi_tilde to Phi_hat
+      Phi_diff0 <- Phi_hat - Phi_tilde
+      delta <- 1
+      while ((ev_bc > ev_restr) & delta > 0){ # delta > 0 is necessary to ensure that Phi_tilde won't get explosive
+        delta <- delta - 0.01
+        Phi_diff <- delta * Phi_diff0
+        Phi_tilde <- Phi_hat - Phi_diff
+        ev_bc <- max(abs(eigen(Phi_tilde)$values))
+      }
     } else{
-    #  # OLS estimates do not satisfy restriction
-    Phi_tilde <- Phi_hat
+      # OLS estimates do not satisfy restriction
+      Phi_tilde <- Phi_hat
     }
-}
-
+  }
 
   return(Phi_tilde)
 }
 
-###################################################################################################################
+###############################################################################################################
 #' Compute the variance-covariance matrix after the bias correction procedure
 #'
 #'@param Phi_tilde Feedback matrix resulting from the bias-correction procedure
@@ -386,32 +329,26 @@ shrink_Phi <- function(Phi_tilde, Phi_hat, ev_restr){
 #'
 #'@keywords internal
 
-
 Get_V_tilde_BC <- function(Phi_tilde, N, RiskFactors, GVARinputs, JLLinputs, FactorLabels, ModelType){
 
-  if(any(ModelType == c("JPS original", "JPS global", "JPS multi"))){
+  if (any(ModelType == c("JPS original", "JPS global", "JPS multi"))){
     T <- nrow(RiskFactors)
-    Xdem <- RiskFactors - matrix(1,T,1)%*%colMeans(RiskFactors)
+    Xdem <- scale(RiskFactors, scale = FALSE)
     resid_tilde <- t(Xdem[2:T, ]) - Phi_tilde %*% t(Xdem[1:(T-1), ])
-    V_tilde <- resid_tilde%*%t(resid_tilde)/(T-1)
-  }
-  else if(any(ModelType == c("GVAR single", "GVAR multi"))){
+    V_tilde <- resid_tilde %*% t(resid_tilde) / (T-1)
+  } else if (any(ModelType == c("GVAR single", "GVAR multi"))){
     Economies <- GVARinputs$Economies # necessary to ensure that the code works for the "GVAR single" model
     GVARinputs$GVARFactors <- DataSet_BS(ModelType, t(RiskFactors), GVARinputs$Wgvar, Economies, FactorLabels)
     V_tilde <- GVAR(GVARinputs, N)$Sigma_y
-  }
-  else if(any(ModelType == c("JLL original", "JLL No DomUnit", "JLL joint Sigma")))
-  {JLLinputs$WishSigmas <- 1
-  JLLPara <- JLL(t(RiskFactors), N, JLLinputs)
-  if (any(ModelType == c("JLL original", "JLL No DomUnit"))){  V_tilde <- JLLPara$Sigmas$VarCov_NonOrtho }
-  if (ModelType == "JLL joint Sigma"){ V_tilde <- JLLPara$Sigmas$VarCov_Ortho}
+  } else if (any(ModelType == c("JLL original", "JLL No DomUnit", "JLL joint Sigma"))){
+    JLLinputs$WishSigmas <- 1
+    JLLPara <- JLL(t(RiskFactors), N, JLLinputs)
+    V_tilde <- if (any(ModelType == c("JLL original", "JLL No DomUnit"))) JLLPara$Sigmas$VarCov_NonOrtho else JLLPara$Sigmas$VarCov_Ortho
   }
 
   return(V_tilde)
 }
 
-
-
 ###############################################################################################################
 #' check whether mean/median of OLS is close to actual OLS estimates
 #'
@@ -432,16 +369,15 @@ Get_V_tilde_BC <- function(Phi_tilde, N, RiskFactors, GVARinputs, JLLinputs, Fac
 Check_comparison__OLS <- function(theta_tilde, theta_OLS, B_check, RiskFactors, N, GVARinputs, JLLinputs,
                                   FactorLabels, Economies, ModelType){
 
-
   cat('... checking closeness of mean/median to actual estimates ... \n')
   Phi_set <- m_var(theta_tilde, B_check, RiskFactors, N, GVARinputs, JLLinputs, FactorLabels, Economies,
                    ModelType, flag_mean = TRUE)
   Phi_new <- Phi_set$Phi_new
   Phi_sample <- Phi_set$Phi_sample  # return sample
-  theta_new <- c(Phi_new)
+  theta_new <- as.vector(Phi_new)
 
-  dist <- sqrt(sum((theta_new - theta_OLS)^2)/length(theta_new))
-  cat(paste('Root mean square distance: ', round(dist,6), "\n"))
+  dist <- sqrt(mean((theta_new - theta_OLS)^2))
+  cat(paste('Root mean square distance: ', round(dist, 6), "\n"))
   cat('... Done! \n\n')
 
   return(dist)
