@@ -19,24 +19,21 @@ Build_xvec <- function(ListInputSet, Economies, FactorLabels, JLLinputs = NULL){
   # 0) Initializing the outputs
   x <- c()
   Dim_x <- c()
-  Name_x <- c()
-
+  Name_x <- vector("character", length(ListInputSet) - 1)
 
   # 1) Build list of interest
-  for (i in 1:(length(ListInputSet) -1)){
-    si <- strfind(ListInputSet[[i]]$Label, ':')
-    Name_x[i] <- Remove_at( substr(ListInputSet[[i]]$Label, start = 1, stop = si-1) ) # remove @ from variables name, if exists
+  for (i in seq_len(length(ListInputSet) - 1)){
+    si <- unlist(gregexpr(":", ListInputSet[[i]]$Label))
+    Name_x[i] <- sub(".*@", "", substr(ListInputSet[[i]]$Label, start = 1, stop = si - 1)) # remove @ from variables name, if exists
 
     Temp_x <- GetAuxPara(ListInputSet[[i]]$Value, ListInputSet[[i]]$Label, ListInputSet[[i]]$LB,
-                         ub= ListInputSet[[i]]$UB, Economies, FactorLabels, JLLinputs)
+                         ub = ListInputSet[[i]]$UB, Economies, FactorLabels, JLLinputs)
 
-
-    x <- rbind(x, t(t(as.vector(Temp_x))) )
-    Dim_x <- rbind(Dim_x, dim(Temp_x) )
+    x <- rbind(x, t(t(as.vector(Temp_x))))
+    Dim_x <- rbind(Dim_x, dim(Temp_x))
   }
 
   Outputs <- list(x0 = x, Dim_x = Dim_x, Name_x = Name_x)
-
   return(Outputs)
 }
 
@@ -64,8 +61,6 @@ Build_xvec <- function(ListInputSet, Economies, FactorLabels, JLLinputs = NULL){
 #' @param FactorLabels   string-list based which contains the labels of all the variables present in the model
 #' @param JLLinputs     list of necessary inputs for the estimation of JLL-based models (see "JLL" function)
 #'
-#' @importFrom pracma tril
-#'
 #'
 #'@keywords internal
 #'
@@ -77,61 +72,31 @@ Build_xvec <- function(ListInputSet, Economies, FactorLabels, JLLinputs = NULL){
 #'  (Euro Area Business Cycle Network Training School - Term Structure Modelling).
 #'  Available at: https://cepr.org/40029
 
-
-
 GetAuxPara <- function(ParaValue, Const_Type_Full, lb, ub, Economies, FactorLabels, JLLinputs = NULL){
 
   Const_Type <- Adjust_Const_Type(Const_Type_Full)
 
   # CASE 1 : Jordan-related constraints
-  if  (any(Const_Type == c("Jordan", "Jordan; stationary", "Jordan MultiCountry", "Jordan MultiCountry; stationary"))){
+  if (grepl("Jordan", Const_Type)){
     a <- Aux_Jordan(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs)
-
   # CASE 2: psd matrix
-  } else if (Const_Type == "psd") { a <- Aux_PSD(ParaValue, Const_Type)
-
+  } else if (Const_Type == "psd") {
+    a <- Aux_PSD(ParaValue, Const_Type)
   # CASE 3: Block diagonal matrix
-  }  else if (Const_Type == "BlockDiag") { a <- Aux_BlockDiag(ParaValue, Const_Type, FactorLabels, Economies)
-
+  } else if (Const_Type == "BlockDiag") {
+    a <- Aux_BlockDiag(ParaValue, Const_Type, FactorLabels, Economies)
   # CASE 4: JLL structure of Sigma matrix
-  }  else if (Const_Type == "JLLstructure"){a <- Aux_JLLstruct(ParaValue, Const_Type, FactorLabels, Economies,
-                                                               JLLinputs)
-
+  } else if (Const_Type == "JLLstructure") {
+    a <- Aux_JLLstruct(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs)
   # CASE 5: 'bounded' and diagonal matrix
-  } else if (any(Const_Type == c("bounded", "diag"))){ a <- Aux_BoundDiag(ParaValue, Const_Type, lb, ub) }
-
-
-  for (j in seq_len(numel(a)) ) {a[j] <- min(max(hablar::s(c(Re(a[j]), -1e20))), 1e20)}
+  } else if (Const_Type %in% c("bounded", "diag")) {
+    a <- Aux_BoundDiag(ParaValue, Const_Type, lb, ub)
+  }
 
+  for (j in seq_len(length(a)) ) {a[j] <- min(max(hablar::s(c(Re(a[j]), -1e20))), 1e20)}
   return(a)
-
 }
 
-##############################################################################################################
-##############################################################################################################
-#' Eliminates the @
-#'
-#' @param s text vector containing the feature of the variable
-#'
-#'
-#' @importFrom pracma strfind isempty
-#'
-#'@references
-#' This function is a modified version of the "killa" function by Le and Singleton (2018). \cr
-#'  "A Small Package of Matlab Routines for the Estimation of Some Term Structure Models." \cr
-#'  (Euro Area Business Cycle Network Training School - Term Structure Modelling).
-#'
-#'@keywords internal
-
-Remove_at <- function(s){
-
-  y <- s
-  ai <- strfind(s, '@')
-
-  if (!isempty(ai) ){ y <- substr(s, start=ai+1, stop= stringr::str_length(s) ) }
-
-  return(y)
-}
 #########################################################################################################
 #'Adjust the constant label
 #'
@@ -140,24 +105,17 @@ Remove_at <- function(s){
 #'@keywords internal
 
 Adjust_Const_Type<- function(Const_Type_Full){
-  # These lines of code ensure that the initial backspace from the constraint name is removed and the
-  # constraint name is perfectly matched.
-  if (contain('Jordan MultiCountry; stationary',Const_Type_Full)) {  Const_Type <- 'Jordan MultiCountry; stationary'
-  } else if (contain('Jordan MultiCountry', Const_Type_Full) ) {  Const_Type <- 'Jordan MultiCountry'
-  } else if (contain('Jordan; stationary', Const_Type_Full) ) { Const_Type <- 'Jordan; stationary'
-  } else if (contain('Jordan', Const_Type_Full) ) { Const_Type <- 'Jordan'
-  } else if (contain('psd', Const_Type_Full) ) { Const_Type <- 'psd'
-  } else if (contain('BlockDiag', Const_Type_Full) ) { Const_Type <- 'BlockDiag'
-  } else if (contain('JLLstructure', Const_Type_Full) ) { Const_Type <- 'JLLstructure'
-  } else if (contain('bounded', Const_Type_Full) ){ Const_Type <- 'bounded'
-  } else if (contain('diag', Const_Type_Full) ){ Const_Type <- 'diag'
+
+  constraints <- c('Jordan MultiCountry; stationary', 'Jordan MultiCountry', 'Jordan; stationary', 'Jordan',
+                   'psd', 'BlockDiag', 'JLLstructure', 'bounded', 'diag')
+  for (constraint in constraints) {
+    if (grepl(constraint, Const_Type_Full)) {
+      return(constraint)
+    }
   }
-
-
-  return(Const_Type)
+  return(Const_Type_Full)
 }
 
-
 ##########################################################################################################
 #' Transformation of the Jordan-related parameters (auxiliary form)
 #'
@@ -169,44 +127,44 @@ Adjust_Const_Type<- function(Const_Type_Full){
 #'
 #'@keywords internal
 
-
 Aux_Jordan <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs){
 
   # 1) SINGLE COUNTRY SETUPS
-  if  (any(Const_Type == c( "Jordan", "Jordan; stationary" ))){
+  if (Const_Type %in% c("Jordan", "Jordan; stationary")){
     K1Q <- ParaValue
-    x <- t(t(eigen(K1Q)$values))
+    x <- eigen(K1Q)$values
 
     # a) Impose stationary restriction (if desired)
-    if ( contain('stationary', Const_Type) ){x <- ImposeStat_Aux(x)   }
+    if (grepl('stationary', Const_Type)) {
+      x <- ImposeStat_Aux(x)
+    }
 
     # b) Compute Jordan form
     idx <- matrix(NaN, nrow=length(x), ncol=1)
     for (j in 1:length(x)){ # Exclude eigenvalues with imaginary values
       if (Im(x[j])==0 ){  idx [j] <- 1} else{  idx [j] <- 0} }
 
-    realx <- Re(x[which(idx==1)])
+    realx <- Re(x[which(idx == 1)])
 
     # Select only the eigenvalues which are purely real
-    if (numel(realx)%%2==0){
+    if (length(realx)%%2==0){
       lQ <- c()
     }else{
       lQ <- realx[1]
       realx <- realx[seq_len(max(0, length(realx) - 1)) + 1]
     }
 
-
-    for (h in seq_len(numel(realx)/2)){
+    for (h in seq_len(length(realx)/2)){
       lQ <- rbind(lQ, 0.5*(realx[2*h-1]+realx[2*h]), (0.5*(realx[2*h-1]- realx[2*h]))^2)
     }
 
     imagx <- t(x[Im(x)!=0]) # Select only the eigenvalues with imaginary values
-    for (h in seq_len(numel(imagx)/2)){ lQ <- rbind(lQ, Re(imagx[2*h-1]), -abs(Im(imagx[2*h-1]))^2)   }
+    for (h in seq_len(length(imagx)/2)){ lQ <- rbind(lQ, Re(imagx[2*h-1]), -abs(Im(imagx[2*h-1]))^2)   }
 
+    if(is.null(dim(lQ))){ lQ <- matrix(lQ) }
 
-    ##############################################
-    # 2) MULTI COUNTRY SETUPS (Jordan MultiCountry)
-      } else if  (any(Const_Type == c("Jordan MultiCountry", "Jordan MultiCountry; stationary"))){
+  # 2) MULTI COUNTRY SETUPS (Jordan MultiCountry)
+  } else if  (any(Const_Type == c("Jordan MultiCountry", "Jordan MultiCountry; stationary"))){
 
     N <- length(FactorLabels$Spanned)
     C <- length(Economies)
@@ -223,8 +181,7 @@ Aux_Jordan <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs
       x <- t(t(eigen(K1Q)$values))
 
       # a) Impose stationary restriction (if desired)
-      if ( contain('stationary', Const_Type) ){ x <- ImposeStat_Aux(x) }
-
+      if ( grepl('stationary', Const_Type) ){ x <- ImposeStat_Aux(x) }
 
       # b) Compute Jordan form
       idx <- matrix(NaN, nrow=length(x), ncol=1)
@@ -234,21 +191,19 @@ Aux_Jordan <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs
       realx <- Re(x[which(idx==1)])
 
       # Select only the eigenvalues which are purely real
-      if (numel(realx)%%2==0){
+      if (length(realx)%%2==0){
         lQ <- c()
       }else{
         lQ <- realx[1]
         realx <- realx[seq_len(max(0, length(realx) - 1)) + 1]
       }
 
-
-      for (h in seq_len(numel(realx)/2)){
+      for (h in seq_len(length(realx)/2)){
         lQ <- rbind(lQ, 0.5*(realx[2*h-1]+realx[2*h]), (0.5*(realx[2*h-1]- realx[2*h]))^2)
-
       }
 
       imagx <- t(x[Im(x)!=0]) # Select only the eignenvalues with imaginary values
-      for (h in seq_len(numel(imagx)/2)){ lQ <- rbind(lQ, Re(imagx[2*h-1]), -abs(Im(imagx[2*h-1]))^2) }
+      for (h in seq_len(length(imagx)/2)){ lQ <- rbind(lQ, Re(imagx[2*h-1]), -abs(Im(imagx[2*h-1]))^2) }
 
       if (i==1) {a <- lQ }else{   a <- rbind(a,lQ) }
 
@@ -261,7 +216,6 @@ Aux_Jordan <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs
   return(lQ)
 }
 
-
 ###########################################################################################################
 #'Impose stationary constraint under the risk-neutral measure
 #'
@@ -269,7 +223,6 @@ Aux_Jordan <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs
 #'
 #'@keywords internal
 
-
 ImposeStat_Aux <- function(yy){
 
   dd <- max(abs(yy))
@@ -278,11 +231,11 @@ ImposeStat_Aux <- function(yy){
   maxdds <- min(max(dd, lb), ub)
   maxdd <- bound2x(maxdds, lb, ub)
 
-  scaled <- maxdd/maxdds
-  yy <- yy*scaled
+  scaled <- maxdd / maxdds
+  yy <- yy * scaled
 
-  if (!isempty(which(is.nan(Im(yy))))){
-    yy[which(is.nan(Im(yy)))] <- Re(yy[which(is.nan(Im(yy)))]) + 0i # replaces the NaN in the imaginary part by 0
+  if (any(is.nan(Im(yy)))){
+    yy[is.nan(Im(yy))] <- Re(yy[is.nan(Im(yy))]) + 0i # replaces the NaN in the imaginary part by 0
   }
   return(yy)
 }
@@ -296,18 +249,13 @@ ImposeStat_Aux <- function(yy){
 #'
 #'@keywords internal
 
-
 Jordan_JLL <- function(ParaValue, C, N){
-  # If the initial guess for the largest eigenvalues of all countries are greater than 1, then the numerical
-  # optimization of the models "JLL original" and "JLL No DomUnit" will crash as the optimization vector will
-  # contain only infinities. Therefore, the code forces one of the eigenvalues to be strictly less than 1.
 
-
-  IDXMaxeigen <- seq(1, N*C, by = N) # identify the indexes of the largest eigenvalue of each country
+  IDXMaxeigen <- seq(1, N * C, by = N) # identify the indexes of the largest eigenvalue of each country
   MaxeigenCS <- diag(ParaValue)[IDXMaxeigen]
 
-  if(all(MaxeigenCS > 0.9999)){
-    BB <- which( min(MaxeigenCS) == MaxeigenCS)
+  if (all(MaxeigenCS > 0.9999)){
+    BB <- which.min(MaxeigenCS)
     ParaValue[IDXMaxeigen[BB], IDXMaxeigen[BB]] <- 0.9998 # Replace the eigenvalue whose value is closest to 1 by 0.9998
   }
 
@@ -321,24 +269,14 @@ Jordan_JLL <- function(ParaValue, C, N){
 #'
 #'@keywords internal
 
-
 Aux_PSD <- function(ParaValue, Const_Type){
 
-  # Preliminary work
-  i <- strfind(Const_Type,'psd')
-  i <- i[1]
-  M <- as.numeric(substr(Const_Type, start= i+3, stop= stringr::str_length(Const_Type) ) )
-  if (is.na(M)) { M <- 1 }
-
   # Compute auxiliary PSD matrix
   N <- dim(ParaValue)[1]
-  mat_psd <- c()
 
-  for (i in 1:M){
-    halfm <- sqrtm_robust(ParaValue[ ,(N*(i-1)+1):(N*i)]) # sqrtm (): computes a matrix square root such that Y=X^(1/2)*X^(1/2).
-    MatOnes<- matrix(1 , nrow= N, ncol = N)
-    mat_psd <- rbind(mat_psd, t(t(halfm[which(tril(MatOnes)==1)])) )
-  }
+  halfm <- sqrtm_robust(ParaValue) # sqrtm (): computes a matrix square root such that Y=X^(1/2)*X^(1/2).
+  MatOnes<- matrix(1 , nrow= N, ncol = N)
+  mat_psd <- t(t(halfm[which(MatOnes == 1 & lower.tri(MatOnes, diag = TRUE))]))
 
   return(mat_psd)
 }
@@ -354,35 +292,22 @@ Aux_PSD <- function(ParaValue, Const_Type){
 #'
 #'@keywords internal
 
-
 Aux_JLLstruct <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinputs){
 
-  # Preliminary work
-  i <- strfind(Const_Type,'JLLstructure')
-  i <- i[1]
-  M <-as.numeric(substr(Const_Type, start= i+13, stop= stringr::str_length(Const_Type) ) )
-  if (is.na(M)) { M<-1 }
-
   # Transform JLL parameters
-  N <- dim(ParaValue)[1]
-  a <- c()
-
   C <- length(Economies)
   Nspa <- length(FactorLabels$Spanned)
   Macro <- length(FactorLabels$Domestic) - Nspa
   G <- length(FactorLabels$Global)
-  K <- C*(Macro+Nspa) + G
+  K <- C*(Macro + Nspa) + G
 
-  ZeroIdxSigmaJLL <- IDXZeroRestrictionsJLLVarCovOrtho(Macro, Nspa, G, Economies, JLLinputs$DomUnit)$VarCovOrtho
+  ZeroIdxSigmaJLL <- IDXZeroRestrictionsJLLVarCovOrtho(Macro, Nspa, G, Economies, JLLinputs$DomUnit)$Sigma_Ye
   MatOnes <- matrix(1, ncol= K, nrow = K)
   MatOnes[ZeroIdxSigmaJLL] <- 0
-  IdxNONzeroSigmaJLL <- which(MatOnes!=0 & tril(MatOnes)==1)
+  IdxNONzeroSigmaJLL <- which(MatOnes!=0 & MatOnes == 1 & lower.tri(MatOnes, diag = TRUE))
 
-
-  for (i in 1:M){
-    halfm <- sqrtm_robust(ParaValue[ ,(N*(i-1)+1):(N*i)]) # sqrtm (): computes a matrix square root such that Y=X^(1/2)*X^(1/2).
-    a <- rbind(a, t(t(halfm[IdxNONzeroSigmaJLL])))
-  }
+  halfm <- sqrtm_robust(ParaValue) # sqrtm (): computes a matrix square root such that Y=X^(1/2)*X^(1/2).
+  a <- t(t(halfm[IdxNONzeroSigmaJLL]))
 
   return(a)
 }
@@ -397,51 +322,51 @@ Aux_JLLstruct <- function(ParaValue, Const_Type, FactorLabels, Economies, JLLinp
 #'
 #'@keywords internal
 
+Aux_BlockDiag <- function(ParaValue, Const_Type, FactorLabels, Economies) {
 
-Aux_BlockDiag <- function(ParaValue, Const_Type, FactorLabels, Economies){
+  # Extract M from Const_Type
+  i <- regexpr("BlockDiag", Const_Type)
+  M <- as.numeric(substr(Const_Type, start = i + 9, stop = nchar(Const_Type)))
+  if (is.na(M)) M <- 1
 
-  # Preliminary work
-  i <- strfind(Const_Type,'BlockDiag')
-  i <- i[1]
-  M <-as.numeric(substr(Const_Type, start= i+9, stop= stringr::str_length(Const_Type) ) )
-  if (is.na(M)) { M <-1 }
-
-  # Compute auxiliary block diagonal matrix
+  # Get dimensions
   G <- length(FactorLabels$Global)
   K <- length(FactorLabels$Domestic)
   C <- length(Economies)
 
+  # Initialize storage
+  a <- numeric(0)
   idx0 <- 0
-  a <- vector(mode="numeric")
+  step <- c(G, rep(K, times = C))
 
-  step <- c(G,rep(K, times=C))
+  # Compute auxiliary block diagonal matrix
+  for (i in seq_len(C + 1)) {
+    idx1 <- idx0 + step[i]
 
-  for (i in 1:(1+C)){
-    idx1 <- idx0+ step[i]
+    if (idx0 < idx1) {
+      seq_indices <- seq(idx0 + 1, idx1)
+      d <- as.matrix(ParaValue[seq_indices, seq_indices])
 
-    if (idx0 < idx1) {seq_indices <- seq(idx0 + 1, idx1)} else {seq_indices <- integer(0) }
-    d <- as.matrix(ParaValue[seq_indices, seq_indices])
-    if (length(d)==0){ atemp <- c() } else{
+      if (length(d) > 0) {
+        N <- nrow(d)
+        Mat1s <- matrix(1, nrow = N, ncol = N)
+        ZeroIdx <- which(d == 0)
 
-      N <- dim(d)[1]
-      atemp <- c()
-      Mat1s <- matrix(1, nrow =N, ncol =N)
-      ZeroIdx <- which(d==0) # Find the indexes of the zero elements (if any)
-      for (i in 1:M){
-        halfm <- sqrtm_robust(d[ ,(N*(i-1)+1):(N*i)]) # sqrtm (): computes a matrix square root such that Y=X^(1/2)*X^(1/2).
-        halfm[ZeroIdx] <- 0 # if there are zero restrictions, ensure that the zeros are properly set after applying "sqrtm_robust".
-        ab <- halfm[which(tril(Mat1s )==1)]
-        ab <- t(t(ab[ab!=0]))
-        atemp<- rbind(atemp, ab)
+        atemp <- do.call(rbind, lapply(seq_len(M), function(i) {
+          halfm <- sqrtm_robust(d[, (N * (i - 1) + 1):(N * i)])
+          halfm[ZeroIdx] <- 0
+          ab <- halfm[lower.tri(Mat1s, diag = TRUE)]
+          matrix(ab[ab != 0], ncol = 1)
+        }))
+
+        a <- c(a, atemp)
       }
     }
-    a <- append(a,atemp)
+
     idx0 <- idx1
   }
 
-  BD_mat <- t(t(a))
-
-  return(BD_mat)
+  return(matrix(a, ncol = 1))
 }
 
 ##########################################################################################################
@@ -457,13 +382,13 @@ Aux_BlockDiag <- function(ParaValue, Const_Type, FactorLabels, Economies){
 Aux_BoundDiag <- function(ParaValue, Const_Type, lb, ub){
 
   # Impose diagonal restriction (if desired)
-  if (contain('diag', Const_Type)) {
-    i <- strfind(Const_Type,'diag')
+  if (grepl('diag', Const_Type)) {
+    i <- unlist(gregexpr("diag", Const_Type))
     i <- i[1]
-    M <-as.numeric(substr(Const_Type, start= i+4, stop= stringr::str_length(Const_Type) ) )
-    if (is.na(M)) { M<-1
+    M <-as.numeric(substr(Const_Type, start= i+4, stop= nchar(Const_Type) ) )
+    if (is.na(M)) { M <- 1
 
-    J <- round(sqrt(numel(ParaValue)/M))
+    J <- round(sqrt(length(ParaValue)/M))
     bb <- matrix(NA, M*J, 1)
     for (i in 1:M){
       if ((i-1)*J+1 < i*J){ IdxSeq <- ((i-1)*J+1):(i*J)}else{ IdxSeq <- numeric()}
@@ -476,7 +401,6 @@ Aux_BoundDiag <- function(ParaValue, Const_Type, lb, ub){
   if (!exists("lb") || is.null(lb) ){ lb <- -Inf }
   if (!exists("ub") || is.null(ub) ){ ub <- Inf }
 
-
   temp <- ParaValue
   temp[] <- lb[]
   lb <- temp
@@ -489,7 +413,7 @@ Aux_BoundDiag <- function(ParaValue, Const_Type, lb, ub){
     d <- as.matrix(ParaValue) # d is a necessary mean to build the variables a and b with correct dimensions (see the loop below)
   }
 
-  if (isempty(ub) || isempty(lb) || (identical(ub,Inf) & identical(lb,Inf))){
+  if (length(ub) == 0 || length(lb) == 0  || (identical(ub,Inf) & identical(lb,Inf))){
     ParaValue <- pmin(hablar::s(c(pmax(hablar::s(c(lb,ParaValue))) , ub)))
   }else{
     ParaValue <- min(hablar::s(c(max(hablar::s(c(lb,ParaValue))) , ub)))
